@@ -100,7 +100,7 @@ def _get_attrs(client_manager, parsed_args):
 
 
 class AddPortToRouter(command.Command):
-    """Add a port to a router"""
+    _description = _("Add a port to a router")
 
     def get_parser(self, prog_name):
         parser = super(AddPortToRouter, self).get_parser(prog_name)
@@ -124,7 +124,7 @@ class AddPortToRouter(command.Command):
 
 
 class AddSubnetToRouter(command.Command):
-    """Add a subnet to a router"""
+    _description = _("Add a subnet to a router")
 
     def get_parser(self, prog_name):
         parser = super(AddSubnetToRouter, self).get_parser(prog_name)
@@ -151,7 +151,7 @@ class AddSubnetToRouter(command.Command):
 
 
 class CreateRouter(command.ShowOne):
-    """Create a new router"""
+    _description = _("Create a new router")
 
     def get_parser(self, prog_name):
         parser = super(CreateRouter, self).get_parser(prog_name)
@@ -222,7 +222,7 @@ class CreateRouter(command.ShowOne):
 
 
 class DeleteRouter(command.Command):
-    """Delete router(s)"""
+    _description = _("Delete router(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteRouter, self).get_parser(prog_name)
@@ -256,10 +256,26 @@ class DeleteRouter(command.Command):
 
 
 class ListRouter(command.Lister):
-    """List routers"""
+    _description = _("List routers")
 
     def get_parser(self, prog_name):
         parser = super(ListRouter, self).get_parser(prog_name)
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_("List routers according to their name")
+        )
+        admin_state_group = parser.add_mutually_exclusive_group()
+        admin_state_group.add_argument(
+            '--enable',
+            action='store_true',
+            help=_("List enabled routers")
+        )
+        admin_state_group.add_argument(
+            '--disable',
+            action='store_true',
+            help=_("List disabled routers")
+        )
         parser.add_argument(
             '--long',
             action='store_true',
@@ -289,6 +305,17 @@ class ListRouter(command.Lister):
             'HA',
             'Project',
         )
+
+        args = {}
+
+        if parsed_args.name is not None:
+            args['name'] = parsed_args.name
+
+        if parsed_args.enable:
+            args['admin_state_up'] = True
+        elif parsed_args.disable:
+            args['admin_state_up'] = False
+
         if parsed_args.long:
             columns = columns + (
                 'routes',
@@ -308,7 +335,7 @@ class ListRouter(command.Lister):
                     'Availability zones',
                 )
 
-        data = client.routers()
+        data = client.routers(**args)
         return (column_headers,
                 (utils.get_item_properties(
                     s, columns,
@@ -317,7 +344,7 @@ class ListRouter(command.Lister):
 
 
 class RemovePortFromRouter(command.Command):
-    """Remove a port from a router"""
+    _description = _("Remove a port from a router")
 
     def get_parser(self, prog_name):
         parser = super(RemovePortFromRouter, self).get_parser(prog_name)
@@ -341,7 +368,7 @@ class RemovePortFromRouter(command.Command):
 
 
 class RemoveSubnetFromRouter(command.Command):
-    """Remove a subnet from a router"""
+    _description = _("Remove a subnet from a router")
 
     def get_parser(self, prog_name):
         parser = super(RemoveSubnetFromRouter, self).get_parser(prog_name)
@@ -368,7 +395,7 @@ class RemoveSubnetFromRouter(command.Command):
 
 
 class SetRouter(command.Command):
-    """Set router properties"""
+    _description = _("Set router properties")
 
     def get_parser(self, prog_name):
         parser = super(SetRouter, self).get_parser(prog_name)
@@ -433,11 +460,19 @@ class SetRouter(command.Command):
             action='store_true',
             help=argparse.SUPPRESS,
         )
-
-        # TODO(tangchen): Support setting 'ha' property in 'router set'
-        # command. It appears that changing the ha state is supported by
-        # neutron under certain conditions.
-
+        routes_ha = parser.add_mutually_exclusive_group()
+        routes_ha.add_argument(
+            '--ha',
+            action='store_true',
+            help=_("Set the router as highly available "
+                   "(disabled router only)")
+        )
+        routes_ha.add_argument(
+            '--no-ha',
+            action='store_true',
+            help=_("Clear high availablability attribute of the router "
+                   "(disabled router only)")
+        )
         # TODO(tangchen): Support setting 'external_gateway_info' property in
         # 'router set' command.
 
@@ -451,6 +486,10 @@ class SetRouter(command.Command):
         attrs = _get_attrs(self.app.client_manager, parsed_args)
 
         # Get the route attributes.
+        if parsed_args.ha:
+            attrs['ha'] = True
+        elif parsed_args.no_ha:
+            attrs['ha'] = False
         if parsed_args.no_route:
             attrs['routes'] = []
         elif parsed_args.clear_routes:
@@ -470,7 +509,7 @@ class SetRouter(command.Command):
 
 
 class ShowRouter(command.ShowOne):
-    """Display router details"""
+    _description = _("Display router details")
 
     def get_parser(self, prog_name):
         parser = super(ShowRouter, self).get_parser(prog_name)
@@ -490,7 +529,7 @@ class ShowRouter(command.ShowOne):
 
 
 class UnsetRouter(command.Command):
-    """Unset router properties"""
+    _description = _("Unset router properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetRouter, self).get_parser(prog_name)
@@ -520,12 +559,11 @@ class UnsetRouter(command.Command):
         if parsed_args.routes:
             try:
                 for route in parsed_args.routes:
+                    route['nexthop'] = route.pop('gateway')
                     tmp_routes.remove(route)
             except ValueError:
                 msg = (_("Router does not contain route %s") % route)
                 raise exceptions.CommandError(msg)
-            for route in tmp_routes:
-                route['nexthop'] = route.pop('gateway')
             attrs['routes'] = tmp_routes
         if attrs:
             client.update_router(obj, **attrs)
